@@ -100,6 +100,68 @@ void overview_coordinate_update(cache_context_t *context) {
   gtk_widget_show_all(context->bearing_hbox);
 }
 
+static void gcvote_set(cache_context_t *context, vote_t *vote) {
+  if(!vote) return;
+
+  if(context->quality) {
+    gtk_widget_destroy(context->quality);
+    context->quality = NULL;
+  } 
+
+  if(context->votes) {
+    gtk_widget_destroy(context->votes);
+    context->votes = NULL;
+  } 
+
+  /* update/draw the voting */
+#ifndef USE_MAEMO
+  GtkTooltips *tips = gtk_tooltips_new();
+#endif
+
+  char *votes_str = g_strdup_printf(_("Quality (%d %s):"), vote->votes,
+				      (vote->votes == 1)?_("vote"):_("votes"));
+  context->votes = GTK_LABEL_SMALL(votes_str);
+  gtk_box_pack_start(GTK_BOX(context->votebox), 
+		     context->votes, FALSE, FALSE, 0);
+  g_free(votes_str);
+  context->quality = icon_get_widget(ICON_STARS, (int)(vote->quality*2-2));
+  gtk_box_pack_start(GTK_BOX(context->votebox), context->quality,
+		     FALSE, FALSE, 0);
+    
+#ifndef USE_MAEMO
+  char *str = g_strdup_printf(_("Quality: %d"), vote->quality);
+  gtk_tooltips_set_tip(tips, context->votebox, str, NULL);
+  g_free(str);
+#endif
+  
+  gtk_widget_show_all(context->votebox);
+
+  g_free(vote);
+}
+
+static void gcvote_callback(vote_t *vote, gpointer data) {
+  cache_context_t *context = (cache_context_t*)data;
+
+  /* no vote returned: request failed, just cleanup */
+  if(!vote) {
+    printf("gcvote: callback for failed request\n"); 
+
+    gcvote_request_free(context->gcvote_request);
+    context->gcvote_request = NULL;
+    return;
+  }
+  
+  printf("gcvote: callback is being called with a %d/%d\n", 
+	 vote->quality, vote->votes);
+  
+  gcvote_set(context, vote);
+
+  gcvote_save(context->appdata, context->cache, &context->gcvote_request->mem);
+
+  gcvote_request_free(context->gcvote_request);
+  context->gcvote_request = NULL;
+}
+
 static GtkWidget *cache_overview(cache_context_t *context) {
   GtkWidget *vbox, *ivbox;
   GtkWidget *table, *tip;
@@ -161,6 +223,9 @@ static GtkWidget *cache_overview(cache_context_t *context) {
     gtk_table_attach(GTK_TABLE(table), ivbox, 1,2,1,2, FALSE,FALSE,0,0);
   }
 
+  /* ----------- vbox containing all ratings ---------- */
+  GtkWidget *ratebox = gtk_vbox_new(FALSE, 0);
+
   /* ----------- box containing difficulty rating ---------- */
   if(cache->difficulty != 0) {
     ivbox = gtk_vbox_new(FALSE, 0);
@@ -170,8 +235,10 @@ static GtkWidget *cache_overview(cache_context_t *context) {
     gtk_box_pack_start(GTK_BOX(ivbox), 
 		  icon_get_widget(ICON_STARS, (int)(cache->difficulty*2-2)),
 		       FALSE, FALSE, 0);
-    gtk_table_attach(GTK_TABLE(table), ivbox, 2,3,0,1, 
-		     GTK_EXPAND | GTK_FILL, 0, GTK_FILL, 0);
+
+    GtkWidget *align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
+    gtk_container_add(GTK_CONTAINER(align), ivbox);
+    gtk_box_pack_start_defaults(GTK_BOX(ratebox), align);
 #ifndef USE_MAEMO
     sprintf(str, _("Difficulty: %.1f"), cache->difficulty);
     gtk_tooltips_set_tip(tips, ivbox, str, NULL);
@@ -186,14 +253,37 @@ static GtkWidget *cache_overview(cache_context_t *context) {
     gtk_box_pack_start(GTK_BOX(ivbox), 
 		       icon_get_widget(ICON_STARS, (int)(cache->terrain*2-2)),
 		       FALSE, FALSE, 0);
-    gtk_table_attach(GTK_TABLE(table), ivbox, 2,3,1,2,
-		     GTK_EXPAND | GTK_FILL, 0, GTK_FILL, 0);
+    GtkWidget *align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
+    gtk_container_add(GTK_CONTAINER(align), ivbox);
+    gtk_box_pack_start_defaults(GTK_BOX(ratebox), align);
 #ifndef USE_MAEMO
     sprintf(str, _("Terrain: %.1f"), cache->terrain);
     gtk_tooltips_set_tip(tips, ivbox, str, NULL);
 #endif
   }
 
+  /* --------------------- GCVote ------------------------ */
+  if(!appdata->disable_gcvote) {
+    vote_t *vote = gcvote_restore(appdata, cache);
+
+    context->gcvote_request = 
+      gcvote_request(appdata, gcvote_callback, cache->url, context);
+
+    context->votebox = gtk_vbox_new(FALSE, 0);
+    GtkWidget *align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
+    gtk_container_add(GTK_CONTAINER(align), context->votebox);
+    gtk_box_pack_start_defaults(GTK_BOX(ratebox), align);
+
+    /* fill with vote if present on disk (will also free vote) */
+    if(vote) 
+      gcvote_set(context, vote);
+
+    gtk_table_attach_defaults(GTK_TABLE(table), ratebox, 2,3,0,2);
+  }
+
+  /* ----------------------------------------------------- */
+
+
   /* ----------------- the two coordinates ----------------- */
   /* ----------------- and the heading/distance ------------ */
   pos_t *refpos = get_pos(appdata);
@@ -223,6 +313,8 @@ static GtkWidget *cache_overview(cache_context_t *context) {
   gtk_box_pack_start(GTK_BOX(vbox), table, 0,0,0);
   gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(),FALSE,FALSE,0);
 
+  /* ----------------------------------------------------- */
+
   if(cache->short_description)
     gtk_box_pack_start_defaults(GTK_BOX(vbox), 
 	html_view(appdata, cache->short_description, 
@@ -357,48 +449,24 @@ static GtkWidget *cache_wpts(appdata_t *appdata, wpt_t *wpt) {
     			      wpt_row+0, wpt_row+1);
     
     /* ------------------ description ------------------------- */
-    if(wpt->desc) {
-      GtkTextBuffer *buffer = gtk_text_buffer_new(NULL);
-      gtk_text_buffer_set_text(buffer, wpt->desc, strlen(wpt->desc));
-
-#ifndef USE_HILDON_TEXT_VIEW
-      GtkWidget *textview = gtk_text_view_new_with_buffer(buffer);
-#else
-      GtkWidget *textview = hildon_text_view_new();
-      hildon_text_view_set_buffer(HILDON_TEXT_VIEW(textview), buffer);
-#endif
-
-      gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textview), GTK_WRAP_WORD);
-      gtk_text_view_set_editable(GTK_TEXT_VIEW(textview), FALSE);
-      gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(textview), FALSE);
-
-      gtk_table_attach_defaults(GTK_TABLE(table), textview, 0,4, 
+    if(wpt->desc) 
+      gtk_table_attach_defaults(GTK_TABLE(table), 
+				simple_text_widget(wpt->desc), 0,4, 
 				wpt_row+1, wpt_row+2);
-    }
 
     /* ------------------ comment ------------------------- */
-    if(wpt->cmt) {
-      GtkTextBuffer *buffer = gtk_text_buffer_new(NULL);
-      gtk_text_buffer_set_text(buffer, wpt->cmt, strlen(wpt->cmt));
-#ifndef USE_HILDON_TEXT_VIEW
-      GtkWidget *textview = gtk_text_view_new_with_buffer(buffer);
-#else
-      GtkWidget *textview = hildon_text_view_new();
-      hildon_text_view_set_buffer(HILDON_TEXT_VIEW(textview), buffer);
-#endif
-      gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textview), GTK_WRAP_WORD);
-      gtk_text_view_set_editable(GTK_TEXT_VIEW(textview), FALSE);
-      gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(textview), FALSE);
-
-      gtk_table_attach_defaults(GTK_TABLE(table), textview, 0,4, 
+    if(wpt->cmt) 
+      gtk_table_attach_defaults(GTK_TABLE(table),
+				simple_text_widget(wpt->cmt), 0,4, 
 				wpt_row+2, wpt_row+3);
-    }
 
     /* --------------------- seperator -------------------------*/
-    if(wpt->next)
+    if(wpt->next) {
+      gtk_table_set_row_spacing(GTK_TABLE(table), wpt_row+2, 8);
       gtk_table_attach_defaults(GTK_TABLE(table), gtk_hseparator_new(), 0,4, 
 				wpt_row+3, wpt_row+4);
-
+      gtk_table_set_row_spacing(GTK_TABLE(table), wpt_row+3, 8);
+    }
 
     wpt_row+=4;
     wpt = wpt->next;
@@ -484,7 +552,8 @@ static void on_gclink_clicked(GtkButton *button, gpointer data) {
 }
 #endif
 
-static GtkWidget *cache_logs(appdata_t *appdata, cache_context_t *context, log_t *log, int is_html) {
+static GtkWidget *cache_logs(appdata_t *appdata, cache_context_t *context, 
+			     log_t *log, int is_html) {
 #ifndef  USE_PANNABLE_AREA
   /* put this inside a scrolled view */
   GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
@@ -500,12 +569,12 @@ static GtkWidget *cache_logs(appdata_t *appdata, cache_context_t *context, log_t
 #define gc_link (FALSE)
 #endif
 
-  GtkWidget *table = gtk_table_new(4*gpx_number_of_logs(log)+(gc_link?1:0), 3, FALSE);
-  int cnt = 0;
+  GtkWidget *vbox = gtk_vbox_new(FALSE, 6);
 
 #ifdef ENABLE_BROWSER_INTERFACE
   if(gc_link) {
-    GtkWidget *but = gtk_button_new_with_label(_("Post a new log entry for this geocache"));
+    GtkWidget *but = 
+      gtk_button_new_with_label(_("Post a new log entry for this geocache"));
 #if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR == 5)
   hildon_gtk_widget_set_theme_size(but, 
 	   (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
@@ -513,26 +582,42 @@ static GtkWidget *cache_logs(appdata_t *appdata, cache_context_t *context, log_t
     gtk_signal_connect(GTK_OBJECT(but), "clicked",
 		       GTK_SIGNAL_FUNC(on_gclink_clicked), context);
 
-    gtk_table_attach_defaults(GTK_TABLE(table), but, 0, 3, 0, 1);
-    cnt++;
+    gtk_box_pack_start_defaults(GTK_BOX(vbox), but);
   }
 #endif
+
+  int logs = gpx_number_of_logs(log);
+  GtkWidget *table = gtk_table_new(2*logs-1, 2,FALSE);
+  int log_cnt = 0;
+
+  gtk_table_set_col_spacing(GTK_TABLE(table), 0, 8);
   
   /* add all logs to the vbox */
   while(log) {
-    gtk_table_attach_defaults(GTK_TABLE(table), gtk_hseparator_new(), 
-			      0, 3, cnt+0, cnt+1);
-#if 0
-    static const char *log_type = "seek/log.aspx";
-    GtkWidget *log_but = 
-      link_icon_button_by_id(appdata, icon_get_widget(ICON_LOG, log->type), 
-			     log_type, log->id);
-    gtk_table_attach(GTK_TABLE(table), log_but, 
-		     0, 1, cnt+1, cnt+2, FALSE, FALSE, 0, 0);
-#else
-    gtk_table_attach_defaults(GTK_TABLE(table), 
-	      icon_get_widget(ICON_LOG, log->type), 0, 1, cnt+1, cnt+2);
-#endif
+    GtkWidget *ivbox = gtk_vbox_new(FALSE, 2);
+    GtkWidget *ihbox = gtk_hbox_new(FALSE, 2);
+
+    static const char *finder_type = "profile/";
+    GtkWidget *finder = link_button_by_id(appdata, log->finder->name, 
+					  finder_type, log->finder->id);
+
+    /* if the finder is a button make sure it's the right size and */
+    /* does not exceed the size limits */
+    if(GTK_WIDGET_TYPE(finder) == GTK_TYPE_BUTTON) {
+#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR == 5)
+      hildon_gtk_widget_set_theme_size(finder, 
+		   (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
+#endif  
+
+      gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(finder))), 
+			      PANGO_ELLIPSIZE_END);
+    } else
+      gtk_label_set_ellipsize(GTK_LABEL(finder), PANGO_ELLIPSIZE_END);
+
+    gtk_box_pack_start(GTK_BOX(ivbox), finder, FALSE, FALSE, 0);
+
+    gtk_box_pack_start_defaults(GTK_BOX(ihbox), 
+	      icon_get_widget(ICON_LOG, log->type));
 
     char date_str[32];
     if(log->day && log->month && log->year) {
@@ -542,37 +627,42 @@ static GtkWidget *cache_logs(appdata_t *appdata, cache_context_t *context, log_t
     } else
       strcpy(date_str, "---");
 
-    gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(date_str),
-			      1, 2, cnt+1, cnt+2);
+    gtk_box_pack_start_defaults(GTK_BOX(ihbox), gtk_label_new(date_str));
 
-    static const char *finder_type = "profile/";
-    GtkWidget *finder = link_button_by_id(appdata, log->finder->name, 
-					  finder_type, log->finder->id);
-
-    gtk_table_attach(GTK_TABLE(table), finder,
-		     2, 3, cnt+1, cnt+2, FALSE, FALSE, 0, 0);
-
-    gtk_table_attach_defaults(GTK_TABLE(table), gtk_hseparator_new(), 
-			      0, 3, cnt+2, cnt+3);
+    gtk_box_pack_start(GTK_BOX(ivbox), ihbox, FALSE, FALSE, 0);
 
+    gtk_table_attach_defaults(GTK_TABLE(table), ivbox, 0, 1,
+			      2*log_cnt, 2*log_cnt+1);
+    
     if(log->text) {
-      gtk_table_attach_defaults(GTK_TABLE(table), 
+      gtk_table_attach_defaults(GTK_TABLE(table),
 	html_view(appdata, log->text, 
 		  is_html?HTML_HTML:HTML_CUSTOM_MARKUP, FALSE, NULL, NULL),
-				0, 3, cnt+3, cnt+4);
+				1, 2, 2*log_cnt, 2*log_cnt+1);    
     }
-    
+
+    if(log_cnt < logs-1) {
+      gtk_table_set_row_spacing(GTK_TABLE(table), 2*log_cnt, 8);
+
+      gtk_table_attach_defaults(GTK_TABLE(table), gtk_hseparator_new(), 
+				0, 2, 2*log_cnt+1, 2*log_cnt+2);
+
+      gtk_table_set_row_spacing(GTK_TABLE(table), 2*log_cnt+1, 8);
+    }
+
     log = log->next;
-    cnt+=4;
+    log_cnt++;
   }
 
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), table);
+
 #ifndef  USE_PANNABLE_AREA
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), 
-					table);
+					vbox);
   return scrolled_window;
 #else
   hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA(pannable_area),
-					 table);
+					 vbox);
   return pannable_area;
 #endif
 }
@@ -632,6 +722,12 @@ static void on_notebook_destroy(GtkWidget *widget, gpointer user_data ) {
 
   printf("destroying notebook\n");
 
+  /* cancel a pending gcvote request */
+  if(context->gcvote_request) {
+    gcvote_request_free(context->gcvote_request);
+    context->gcvote_request = NULL;
+  }
+
   notes_destroy_event(NULL, context);
   goto_destroy_event(NULL, context);
 
@@ -643,6 +739,98 @@ static void on_notebook_destroy(GtkWidget *widget, gpointer user_data ) {
   free(user_data);
 }
 
+#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
+#define CUSTOM_NOTEBOOK
+#endif
+
+static GtkWidget *notebook_new(void) {
+#ifdef CUSTOM_NOTEBOOK
+  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
+
+  GtkWidget *notebook =  gtk_notebook_new();
+
+  /* prevents user from accidentially touching the breadcrumb trail */
+  /* (looks ugly on fremantle as it isn't themed) */
+  //  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_BOTTOM);
+
+  /* solution for fremantle: we use a row of ordinary buttons instead */
+  /* of regular tabs */
+
+  /* hide the regular tabs */
+  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
+
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), notebook);
+
+  /* store a reference to the notebook in the vbox */
+  g_object_set_data(G_OBJECT(vbox), "notebook", (gpointer)notebook);
+
+  /* create a hbox for the buttons */
+  GtkWidget *hbox = gtk_hbox_new(TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(vbox), "hbox", (gpointer)hbox);
+
+  return vbox;
+#else
+  return gtk_notebook_new();
+#endif
+}
+
+static void on_notebook_button_clicked(GtkWidget *button, gpointer data) {
+  GtkNotebook *nb = 
+    GTK_NOTEBOOK(g_object_get_data(G_OBJECT(data), "notebook"));
+
+  gint page = (gint)g_object_get_data(G_OBJECT(button), "page");
+  gtk_notebook_set_current_page(nb, page);
+}
+
+static void notebook_append_page(GtkWidget *notebook, 
+				 GtkWidget *page, char *label) {
+#ifdef CUSTOM_NOTEBOOK
+  GtkNotebook *nb = 
+    GTK_NOTEBOOK(g_object_get_data(G_OBJECT(notebook), "notebook"));
+
+  gint page_num = gtk_notebook_append_page(nb, page, gtk_label_new(label));
+  GtkWidget *button = NULL;
+
+  /* select button for page 0 by default */
+  if(!page_num) {
+    button = gtk_radio_button_new_with_label(NULL, label);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
+    g_object_set_data(G_OBJECT(notebook), "group_master", (gpointer)button);
+  } else {
+    GtkWidget *master = g_object_get_data(G_OBJECT(notebook), "group_master");
+    button = gtk_radio_button_new_with_label_from_widget(
+				 GTK_RADIO_BUTTON(master), label);
+  }
+
+  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(button), FALSE);
+  g_object_set_data(G_OBJECT(button), "page", (gpointer)page_num);
+
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+	   GTK_SIGNAL_FUNC(on_notebook_button_clicked), notebook);
+
+#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR == 5)
+  hildon_gtk_widget_set_theme_size(button, 
+	   (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
+#endif  
+
+  gtk_box_pack_start_defaults(
+     GTK_BOX(g_object_get_data(G_OBJECT(notebook), "hbox")),
+     button);
+
+#else
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, gtk_label_new(label));
+#endif
+}
+
+static GObject *notebook_object(GtkWidget *notebook) {
+#ifdef CUSTOM_NOTEBOOK
+  return G_OBJECT(g_object_get_data(G_OBJECT(notebook), "notebook")); 
+#else
+  return G_OBJECT(notebook);
+#endif
+}
+
 GtkWidget *cache_view(appdata_t *appdata, cache_t *cache) {
   GtkWidget *notebook;
 
@@ -659,49 +847,42 @@ GtkWidget *cache_view(appdata_t *appdata, cache_t *cache) {
 #define TAB_WPTS   _("Waypoints")
 #endif
 
-  notebook =  gtk_notebook_new();
+  notebook = notebook_new();
 
-#ifdef USE_MAEMO
-#if MAEMO_VERSION_MAJOR >= 5
-  /* prevents user from accidentially touching the breadcrumb trail */
-  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_BOTTOM);
-#endif
-#endif
-
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
-       cache_overview(cache_context), gtk_label_new(_("Main")));
+  notebook_append_page(notebook, 
+       cache_overview(cache_context), _("Main"));
 
   if(cache->long_description)
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
-	   cache_description(appdata, cache), gtk_label_new(TAB_DESC));
+    notebook_append_page(notebook, 
+	   cache_description(appdata, cache), TAB_DESC);
 
   if(cache->hint)
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
-	   cache_hint(appdata, cache), gtk_label_new(_("Hint")));
+    notebook_append_page(notebook, 
+	   cache_hint(appdata, cache), _("Hint"));
 
   if(cache->log)
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
+    notebook_append_page(notebook, 
      cache_logs(appdata, cache_context, cache->log, cache->logs_are_html), 
-	   gtk_label_new(_("Logs")));
+	   _("Logs"));
 
   if(cache->wpt)
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
-	     cache_wpts(appdata, cache->wpt), gtk_label_new(TAB_WPTS));
+    notebook_append_page(notebook, 
+	     cache_wpts(appdata, cache->wpt), TAB_WPTS);
 
   if(cache->tb)
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
-	     cache_tbs(appdata, cache->tb), gtk_label_new(_("TBs")));
+    notebook_append_page(notebook, 
+	     cache_tbs(appdata, cache->tb), _("TBs"));
 
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
-	   cache_notes(cache_context), gtk_label_new(_("Notes")));
+  notebook_append_page(notebook, 
+	   cache_notes(cache_context), _("Notes"));
 
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
-	   goto_cache(cache_context), gtk_label_new(_("Goto")));
+  notebook_append_page(notebook, 
+	   goto_cache(cache_context), _("Goto"));
 
-  g_signal_connect(G_OBJECT(notebook), "switch-page", 
+  g_signal_connect(notebook_object(notebook), "switch-page", 
 	   G_CALLBACK(on_notebook_page_change), cache_context);
 
-  g_signal_connect(G_OBJECT(notebook), "destroy", 
+  g_signal_connect(notebook_object(notebook), "destroy", 
 	   G_CALLBACK(on_notebook_destroy), cache_context);
 
   return notebook;
