@@ -41,7 +41,7 @@
 
 #define DEFAULT_IMAGE_PATH_HOME  ".gpxview/"
 
-#define LOCALEDIR "/usr/share/locale"
+#define LOCALEDIR PREFIX "/share/locale"
 #define PACKAGE   "gpxview"
 
 // for osm-gps-map:
@@ -54,13 +54,13 @@
 
 #ifdef USE_MAEMO
 #define BIG_BALLOONS
-#define BALLOON_AREA_WIDTH       380
-#define BALLOON_AREA_HEIGHT      100
-#define BALLOON_CORNER_RADIUS     20
-#define OSD_FONT_SIZE             28
-#define OSD_DIAMETER              60
-#define OSD_SCALE_FONT_SIZE       20
-#define OSD_COORDINATES_FONT_SIZE 20
+#define BALLOON_AREA_WIDTH        (380)
+#define BALLOON_AREA_HEIGHT       (100)
+#define BALLOON_CORNER_RADIUS     (20)
+#define OSD_FONT_SIZE             (28.0)
+#define OSD_DIAMETER              (60)
+#define OSD_SCALE_FONT_SIZE       (20.0)
+#define OSD_COORDINATES_FONT_SIZE (20.0)
 
 #endif
 
