@@ -32,6 +32,8 @@
 #include <errno.h>
 #endif
 
+static void gps_unregister_all(appdata_t *appdata);
+
 #ifndef ENABLE_LIBLOCATION
 
 /* maybe user configurable later on ... */
@@ -114,13 +116,18 @@ static int gps_connect(gps_state_t *gps_state) {
 #ifdef USE_MAEMO
   char errstr[256] = "";
 
+  if(!gps_state) {
+    printf("No gps state\n");
+    return -1;
+  }
+
   /* We need to start gpsd (via gpsbt) first. */
   memset(&gps_state->context, 0, sizeof(gpsbt_t));
   errno = 0;
 
   if(gpsbt_start(NULL, 0, 0, 0, errstr, sizeof(errstr),
 		 0, &gps_state->context) < 0) {
-    printf("Error connecting to GPS receiver: (%d) %s (%s)\n",
+    printf("gps: Error connecting to GPS receiver: (%d) %s (%s)\n",
 	   errno, strerror(errno), errstr);
   }
 #endif
@@ -134,39 +141,39 @@ static int gps_connect(gps_state_t *gps_state) {
   while(retries && 
 	(GNOME_VFS_OK != (vfs_result = gnome_vfs_inet_connection_create(
 		&gps_state->iconn, GPSD_HOST, GPSD_PORT, NULL)))) {
-    printf("Error creating connection to GPSD, retrying ...\n");
+    printf("gps: Error creating connection to GPSD, retrying ...\n");
 
     retries--;
     sleep(1);
   }
 
   if(!retries) {
-    printf("Finally failed ...\n");
+    printf("gps: Finally failed ...\n");
     return -1;
   }
 
   retries = 5;
   while(retries && ((gps_state->socket = 
      gnome_vfs_inet_connection_to_socket(gps_state->iconn)) == NULL)) {
-    printf("Error creating connecting GPSD socket, retrying ...\n");
+    printf("gps: Error creating connecting GPSD socket, retrying ...\n");
 
     retries--;
     sleep(1);
   }
 
   if(!retries) {
-    printf("Finally failed ...\n");
+    printf("gps: Finally failed ...\n");
     return -1;
   }
 
   GTimeVal timeout = { 10, 0 };
   if(GNOME_VFS_OK != (vfs_result = gnome_vfs_socket_set_timeout(
 	gps_state->socket, &timeout, NULL))) {
-    printf("Error setting GPSD timeout\n");
+    printf("gps: Error setting GPSD timeout\n");
     return -1;
   } 
 
-  printf("GPSD connected ...\n");
+  printf("gps: GPSD connected ...\n");
 
   return 0;
 }	
@@ -371,7 +378,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata) {
 	    for (j = 0; j < gpsdata->satellites; j++) {
 	      PRN[j]=elevation[j]=azimuth[j]=ss[j]=used[j]=0;
 	    }
-	    //	    printf("sats = %d\n", gpsdata->satellites);
+	    //	    printf("gps: sats = %d\n", gpsdata->satellites);
 	    for (j = 0, gpsdata->satellites_used = 0; j < gpsdata->satellites; j++) {
 	      if ((sp != NULL) && ((sp = strchr(sp, ':')) != NULL)) {
 		sp++;
@@ -399,6 +406,23 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata) {
   }
 }
 
+/* call one of the application provided gps callbacks */
+static void do_app_cb(gpointer data, gpointer user_data) {
+  appdata_t *appdata = (appdata_t*)user_data;
+  gps_cb_t *cb = (gps_cb_t*)data;
+  cb->cb(appdata->gps_state, cb->data);
+}
+
+/* walk though list of all application provided callbacks */
+static gboolean gps_idle_cb(gpointer data) {
+  appdata_t *appdata = (appdata_t*)data;
+  printf("gps: idle callback, calling app callbacks\n");
+
+  g_slist_foreach(appdata->gps_state->cb, do_app_cb, appdata);
+
+  return FALSE;
+}
+
 gpointer gps_thread(gpointer data) {
   GnomeVFSFileSize bytes_read;
   GnomeVFSResult vfs_result;
@@ -416,7 +440,7 @@ gpointer gps_thread(gpointer data) {
   while(1) {
     if(appdata->use_gps) {
       if(!connected) {
-	printf("trying to connect\n");
+	printf("gps: trying to connect\n");
 
 	if(gps_connect(appdata->gps_state) < 0)
 	  sleep(10);
@@ -442,7 +466,7 @@ gpointer gps_thread(gpointer data) {
 	    if(vfs_result == GNOME_VFS_OK) {
 	      str[bytes_read] = 0; 
 	    
-	      //	  printf("msg: %s (%d)\n", str, strlen(str));
+	      //	  printf("gps: msg: %s (%d)\n", str, strlen(str));
 	      
 	      g_mutex_lock(appdata->gps_state->mutex);
 	      
@@ -452,6 +476,7 @@ gpointer gps_thread(gpointer data) {
 	    
 	      gps_unpack(str, &appdata->gps_state->gpsdata);
 	      g_mutex_unlock(appdata->gps_state->mutex);
+	      g_idle_add(gps_idle_cb, appdata); 
 	    }
 	  }
 	}
@@ -459,7 +484,7 @@ gpointer gps_thread(gpointer data) {
       }
     } else {
       if(connected) {
-	printf("stopping GPS connection due to user request\n");
+	printf("gps: stopping GPS connection due to user request\n");
 	gnome_vfs_inet_connection_destroy(appdata->gps_state->iconn, NULL);
 
 #ifdef USE_MAEMO
@@ -471,7 +496,7 @@ gpointer gps_thread(gpointer data) {
     }
   }
 
-  printf("GPS thread ended???\n");
+  printf("gps: thread ended???\n");
   return NULL;
 }
 
@@ -485,6 +510,7 @@ void gps_init(appdata_t *appdata) {
 }
 
 void gps_release(appdata_t *appdata) { 
+  gps_unregister_all(appdata);
 #ifdef USE_MAEMO
   gpsbt_stop(&appdata->gps_state->context);
 #endif
@@ -540,11 +566,11 @@ location_changed(LocationGPSDevice *device, gps_state_t *gps_state) {
 void gps_init(appdata_t *appdata) {
   gps_state_t *gps_state = appdata->gps_state = g_new0(gps_state_t, 1);
 
-  printf("GPS init: Using liblocation\n");
+  printf("gps: init: Using liblocation\n");
 
   gps_state->device = g_object_new(LOCATION_TYPE_GPS_DEVICE, NULL);  
   if(!gps_state->device) {
-    printf("Unable to connect to liblocation\n");
+    printf("gps: Unable to connect to liblocation\n");
     return;
   }
 
@@ -560,13 +586,14 @@ void gps_init(appdata_t *appdata) {
 #endif
      ) {
 
-    printf("Having control over GPSD and GPS is to be enabled, starting it\n");
+    printf("gps: Having control over GPSD and GPS is to be enabled, starting it\n");
     location_gpsd_control_start(gps_state->control);
   }
 }
 
 void gps_release(appdata_t *appdata) {
   gps_state_t *gps_state = appdata->gps_state;
+  gps_unregister_all(appdata);
   
   if(!gps_state->device) return;
 
@@ -575,7 +602,7 @@ void gps_release(appdata_t *appdata) {
      && gps_state->control->can_control
 #endif     
      ) {
-    printf("Having control over GPSD, stopping it\n");
+    printf("gps: Having control over GPSD, stopping it\n");
     location_gpsd_control_stop(gps_state->control);
   }
   
@@ -644,11 +671,44 @@ gps_sat_t *gps_get_sats(appdata_t *appdata) {
 #endif
 
 void *gps_register_callback(appdata_t *appdata, gps_cb cb, gpointer data) {
-  printf("register gps callback\n");
-  return NULL;
+  printf("gps: register gps callback\n");
+
+  if(!appdata->gps_state)
+    return NULL;
+
+  /* allocate callback info strcuture */
+  gps_cb_t *cb_info = g_new0(gps_cb_t, 1);
+  cb_info->cb = cb;
+  cb_info->data = data;
+
+  /* and insert it into list of callbacks */
+  appdata->gps_state->cb = g_slist_append(appdata->gps_state->cb, cb_info);
+
+  return cb_info;
 }
 
 
 void gps_unregister_callback(appdata_t *appdata, void *cb) {
-  printf("unregister gps callback\n");
+  printf("gps: unregister gps callback\n");
+
+  if(!appdata->gps_state)
+    return;
+
+  /* the item must be in the list */
+  g_assert(g_slist_find(appdata->gps_state->cb, cb));
+    
+  g_free(cb);
+  appdata->gps_state->cb = g_slist_remove(appdata->gps_state->cb, cb);
+}
+
+static void gps_unregister_all(appdata_t *appdata) {
+  printf("gps: unregister all callbacks: ");
+
+  while(appdata->gps_state->cb) {
+    printf(".");
+    g_free(appdata->gps_state->cb->data);
+    appdata->gps_state->cb = g_slist_remove(appdata->gps_state->cb,
+			    appdata->gps_state->cb->data);
+  }
+  printf("\n");
 }
