@@ -1329,6 +1329,17 @@ int gpx_total_caches(gpx_t *gpx) {
   return num;
 }
 
+/* return number of caches in all gpx files */
+int gpx_total_caches_global(gpx_t *gpx) {
+  int num = 0;
+  while(gpx) {
+    num += gpx_total_caches(gpx);
+    gpx = gpx->next;
+  }
+
+  return num;
+}
+
 int gpx_number_of_waypoints(wpt_t *wpt) {
   int num = 0;
   
