@@ -20,8 +20,10 @@
 #ifndef HTML_H
 #define HTML_H
 
+typedef enum { HTML_PLAIN_TEXT, HTML_HTML, HTML_CUSTOM_MARKUP } html_mode_t;
+
 GtkWidget *html_view(appdata_t *appdata, char *text, 
-		     gboolean is_html, gboolean scrollwin,
+		     html_mode_t mode, gboolean scrollwin,
 		     cache_t *cache, char *anchor);
 void html_zoom(appdata_t *appdata, gboolean in);
 void html_copy_to_clipboard(appdata_t *appdata);
