@@ -24,19 +24,20 @@ struct icon_data {
   GdkPixbuf **data;
   int count;
 } icons[] = {
-  { NULL, CACHE_TYPE_MAX+1 },  /* ICON_CACHE_TYPE */
-  { NULL, CACHE_CONT_MAX+1 },  /* ICON_CACHE_SIZE */
-  { NULL, 9 },                 /* ICON_STARS      */
-  { NULL, LOG_TYPE_MAX+1 },    /* ICON_LOG        */
-  { NULL, 9 },                 /* ICON_HEADING    */
-  { NULL, WPT_SYM_MAX+1 },     /* ICON_WPT        */
-  { NULL, 1 },                 /* ICON_TB         */
-  { NULL, 9 },                 /* ICON_MISC       */
-  { NULL, 3 },                 /* ICON_FILE       */
+  { NULL, CACHE_TYPE_MAX+1 },  /* ICON_CACHE_TYPE      */
+  { NULL, CACHE_TYPE_MAX+1 },  /* ICON_CACHE_TYPE_SEMI */
+  { NULL, CACHE_CONT_MAX+1 },  /* ICON_CACHE_SIZE      */
+  { NULL, 9 },                 /* ICON_STARS           */
+  { NULL, LOG_TYPE_MAX+1 },    /* ICON_LOG             */
+  { NULL, 9 },                 /* ICON_HEADING         */
+  { NULL, WPT_SYM_MAX+1 },     /* ICON_WPT             */
+  { NULL, 1 },                 /* ICON_TB              */
+  { NULL, 9 },                 /* ICON_MISC            */
+  { NULL, 3 },                 /* ICON_FILE            */
   { NULL, -1 }
 };
 
-/* ICON_CACHE_TYPE */
+/* ICON_CACHE_TYPE / ICON_CACHE_TYPE_SEMI */
 const char *cache_type_icon_name[] = {
   "traditional", "multi",      "mystery", "virtual", "webcam", "event",
   "letterbox",   "earthcache", "wherigo", "megaevent", "cito"
@@ -154,6 +155,9 @@ void icons_init(void) {
   /* load cache type icons */
   icons_load(ICON_CACHE_TYPE, "cache_type_%s.%s", cache_type_icon_name);
 
+  /* load cache type icons */
+  icons_load(ICON_CACHE_TYPE_SEMI, "cache_type_%s_semi.%s", cache_type_icon_name);
+
   /* load cache container/size icons */
   icons_load(ICON_CACHE_SIZE, "cache_size_%s.%s", cache_size_icon_name);
 
