@@ -19,6 +19,8 @@
 #include <string.h>
 #include <math.h>
 
+#include <curl/curl.h>
+
 #include <time.h>
 #include <sys/time.h>
 
@@ -29,6 +31,10 @@
 
 #ifdef USE_MAEMO
 #include <hildon/hildon-banner.h>
+#if MAEMO_VERSION_MAJOR >= 5
+#include <hildon/hildon-note.h>
+#include <hildon/hildon-check-button.h>
+#endif
 #endif
 
 #include <locale.h>
@@ -92,17 +98,18 @@ void errorf(const char *fmt, ...) {
     }
   }
 
+#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
   GtkWidget *dialog = gtk_message_dialog_new(
 			   GTK_WINDOW(NULL),
 			   GTK_DIALOG_DESTROY_WITH_PARENT,
-#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
 			   GTK_MESSAGE_ERROR, 
-#else
-			   GTK_MESSAGE_OTHER, 			   
-#endif
 			   GTK_BUTTONS_CLOSE, buf);
 
   gtk_window_set_title(GTK_WINDOW(dialog), _("ERROR"));
+#else
+  GtkWidget *dialog = 
+    hildon_note_new_information(GTK_WINDOW(NULL), buf);
+#endif
 
   gtk_dialog_run(GTK_DIALOG(dialog));
   gtk_widget_destroy(dialog);
@@ -774,16 +781,12 @@ static void gpxlist_view_onRowActivated(GtkTreeView *treeview,
     if(col_name && !strcmp(col_name, "Del")) {
       printf("clicked delete\n");
 
+#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
       /* ask user what he wants */
       GtkWidget *dialog = gtk_message_dialog_new(
   	         GTK_WINDOW(appdata->window),
 		 GTK_DIALOG_DESTROY_WITH_PARENT,
-		 /* this doesn't fix the text selection issues ... */
-#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >=5)
-		 GTK_MESSAGE_OTHER,
-#else
 		 GTK_MESSAGE_QUESTION,
-#endif
 		 GTK_BUTTONS_CANCEL,
 		 _("Do you want to close this entry only or do "
 		   "you want to remove it completely from the list?"));
@@ -793,11 +796,44 @@ static void gpxlist_view_onRowActivated(GtkTreeView *treeview,
 			     _("Close"), 2,
 			     NULL);
 
+      gtk_window_set_title(GTK_WINDOW(dialog), _("Close or remove entry?")); 
+#else
+
+      GtkWidget *dialog = 
+	gtk_dialog_new_with_buttons(_("Close or remove entry?"),
+			    GTK_WINDOW(appdata->window),
+			    GTK_DIALOG_DESTROY_WITH_PARENT,
+			    GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+			    _("Remove"), 1,
+			    _("Close"), 2,
+			    NULL);
+				    
+      GtkWidget *content_area = 
+	gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+
+      GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+
+      gtk_box_pack_start(GTK_BOX(hbox),
+	 gtk_image_new_from_stock( GTK_STOCK_DIALOG_QUESTION, 
+				   GTK_ICON_SIZE_DIALOG),
+			 FALSE, FALSE, 0);
+
+      GtkWidget *label = gtk_label_new(
+		  _("Do you want to close this entry only or do "
+		    "you want to remove it completely from the list?"));
+      
+      gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+      gtk_label_set_line_wrap_mode(GTK_LABEL(label), PANGO_WRAP_WORD);
+
+      gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
+      gtk_container_add (GTK_CONTAINER (content_area), hbox);
+
+      gtk_widget_show_all (dialog);
+#endif
+
       if(gpx->closed)
 	gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog), 2, FALSE);
 
-      gtk_window_set_title(GTK_WINDOW(dialog), _("Close or remove entry?"));
-      
       /* set the active flag again if the user answered "no" */
       switch(gtk_dialog_run(GTK_DIALOG(dialog))) {
 
@@ -1092,6 +1128,7 @@ cb_menu_about(GtkWidget *window, gpointer data) {
   const gchar *authors[] = {
     "Till Harbaum <till@harbaum.org>",
     "John Stowers <john.stowers@gmail.com>",
+    "GCVote: Guido Wegener <guido.wegener@gmx.de>",
     NULL };
 
   context.dialog = g_object_new(GTK_TYPE_ABOUT_DIALOG,
@@ -1300,8 +1337,8 @@ gpx_t *search_do(appdata_t *appdata, gpx_t *gpx, char *phrase,
       else if(cache->long_description && (what & SEARCH_DESC) && 
 	      strcasestr(cache->long_description, phrase))
 	hit = 1;
-      else if(cache->owner && (what & SEARCH_OWNER) && 
-	      strcasestr(cache->owner, phrase))
+      else if(cache->owner && cache->owner->name && (what & SEARCH_OWNER) && 
+	      strcasestr(cache->owner->name, phrase))
 	hit = 1;
       
       if(hit) {
@@ -1330,11 +1367,38 @@ typedef struct {
   GtkWidget *in_id, *in_name, *in_desc, *in_owner, *in_finds;
 } search_context_t;
 
+
+static GtkWidget *check_button_new_with_label(char *label) {
+#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
+  return gtk_check_button_new_with_label(label);
+#else
+  GtkWidget *cbut = 
+    hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
+  gtk_button_set_label(GTK_BUTTON(cbut), label);
+  return cbut;
+#endif
+}
+
+static void check_button_set_active(GtkWidget *button, gboolean active) {
+#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), active);
+#else
+  hildon_check_button_set_active(HILDON_CHECK_BUTTON(button), active);
+#endif
+}
+
+static gboolean check_button_get_active(GtkWidget *button) {
+#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
+  return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
+#else
+  return hildon_check_button_get_active(HILDON_CHECK_BUTTON(button));
+#endif
+}
+
 static void callback_finds_toggled(GtkWidget *widget, gpointer data ) {
   search_context_t *context = (search_context_t*)data;
 
-  gboolean in_finds = gtk_toggle_button_get_active(
-	  GTK_TOGGLE_BUTTON(context->in_finds));
+  gboolean in_finds = check_button_get_active(context->in_finds);
 
   gtk_widget_set_sensitive(context->entry,    !in_finds);
   gtk_widget_set_sensitive(context->in_id,    !in_finds);
@@ -1364,24 +1428,20 @@ cb_menu_search(GtkWidget *window, gpointer data) {
   GtkWidget *table = gtk_table_new(2, 2, TRUE);
   gtk_table_set_col_spacing(GTK_TABLE(table), 0, 8);
 
-  context.in_id = gtk_check_button_new_with_label(_("Waypoint IDs"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context.in_id), 
-			       appdata->search & SEARCH_ID);
+  context.in_id = check_button_new_with_label(_("Waypoint IDs"));
+  check_button_set_active(context.in_id, appdata->search & SEARCH_ID);
   gtk_table_attach_defaults(GTK_TABLE(table), context.in_id, 0, 1, 0, 1);
 
-  context.in_name = gtk_check_button_new_with_label(_("Names"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context.in_name), 
-			       appdata->search & SEARCH_NAME);
+  context.in_name = check_button_new_with_label(_("Names"));
+  check_button_set_active(context.in_name, appdata->search & SEARCH_NAME);
   gtk_table_attach_defaults(GTK_TABLE(table), context.in_name, 1, 2, 0, 1);
 
-  context.in_desc = gtk_check_button_new_with_label(_("Descriptions"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context.in_desc), 
-			       appdata->search & SEARCH_DESC);
+  context.in_desc = check_button_new_with_label(_("Descriptions"));
+  check_button_set_active(context.in_desc, appdata->search & SEARCH_DESC);
   gtk_table_attach_defaults(GTK_TABLE(table), context.in_desc, 0, 1, 1, 2);
 
-  context.in_owner = gtk_check_button_new_with_label(_("Owner"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context.in_owner), 
-			       appdata->search & SEARCH_OWNER);
+  context.in_owner = check_button_new_with_label(_("Owner"));
+  check_button_set_active(context.in_owner, appdata->search & SEARCH_OWNER);
   gtk_table_attach_defaults(GTK_TABLE(table), context.in_owner, 1, 2, 1, 2);
 
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), table);
@@ -1402,9 +1462,8 @@ cb_menu_search(GtkWidget *window, gpointer data) {
 
   GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
 
-  context.in_finds = gtk_check_button_new_with_label(_("Search finds for"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context.in_finds), 
-			       appdata->search & SEARCH_FINDS);
+  context.in_finds = check_button_new_with_label(_("Search finds for"));
+  check_button_set_active(context.in_finds, appdata->search & SEARCH_FINDS);
   gtk_box_pack_start_defaults(GTK_BOX(hbox), context.in_finds);
   g_signal_connect(G_OBJECT(context.in_finds), "toggled",
 		   G_CALLBACK(callback_finds_toggled), &context);
@@ -1444,27 +1503,27 @@ cb_menu_search(GtkWidget *window, gpointer data) {
 		    HILDON_NUMBER_EDITOR(context.spinner));
 #endif
 
-    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context.in_finds)))
+    if(check_button_get_active(context.in_finds))
       appdata->search |=  SEARCH_FINDS;
     else
       appdata->search &= ~SEARCH_FINDS;
 
-    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context.in_id)))
+    if(check_button_get_active(context.in_id))
       appdata->search |=  SEARCH_ID;
     else
       appdata->search &= ~SEARCH_ID;
 
-    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context.in_name)))
+    if(check_button_get_active(context.in_name))
       appdata->search |=  SEARCH_NAME;
     else
       appdata->search &= ~SEARCH_NAME;
 
-    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context.in_desc)))
+    if(check_button_get_active(context.in_desc))
       appdata->search |=  SEARCH_DESC;
     else
       appdata->search &= ~SEARCH_DESC;
  
-    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context.in_owner)))
+    if(check_button_get_active(context.in_owner))
       appdata->search |=  SEARCH_OWNER;
     else
       appdata->search &= ~SEARCH_OWNER;
@@ -2295,6 +2354,8 @@ int main(int argc, char *argv[]) {
 
   gtk_init (&argc, &argv);
 
+  curl_global_init(CURL_GLOBAL_ALL);
+
 #ifdef USE_MAEMO
   printf("Installing osso context for \"org.harbaum." APP "\"\n");
   appdata.osso_context = osso_initialize("org.harbaum."APP, 
