@@ -252,6 +252,7 @@ osd_render_balloon(osm_gps_map_osd_t *osd) {
     priv->balloon.rect.w = x1 - x0 - 2*BALLOON_BORDER;
     priv->balloon.rect.h = y1 - y0 - 2*BALLOON_BORDER;
 
+    g_assert(priv->balloon.surface);
     cairo_t *cr = cairo_create(priv->balloon.surface);
     cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
     cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.0);
@@ -776,6 +777,7 @@ osd_render_source_sel(osm_gps_map_osd_t *osd, gboolean force_rerender) {
 #endif
 
     /* draw source selector */
+    g_assert(priv->source_sel.surface);
     cairo_t *cr = cairo_create(priv->source_sel.surface);
     cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
     cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.0);
@@ -826,6 +828,7 @@ osd_source_reallocate(osm_gps_map_osd_t *osd) {
         cairo_text_extents_t extents;
 
         /* determine content size */
+        g_assert(priv->source_sel.surface);
         cairo_t *cr = cairo_create(priv->source_sel.surface);
         cairo_select_font_face (cr, "Sans",
                                 CAIRO_FONT_SLANT_NORMAL,
@@ -1212,15 +1215,24 @@ osd_render_centered_text(cairo_t *cr, int y, int width, char *text) {
     strcpy(p, text);
 
     cairo_text_extents_t extents;
+    memset(&extents, 0, sizeof(cairo_text_extents_t));
     cairo_text_extents (cr, p, &extents);
     g_assert(extents.width != 0.0);
 
     /* check if text needs to be truncated */
-    int trunc_at = strlen(text)-1;
+    int trunc_at = strlen(text);
     while(extents.width > width) {
-        g_assert(trunc_at > 0);
+
+        /* cut off all utf8 multibyte remains so the actual */
+        /* truncation only deals with one byte */
+        while((p[trunc_at-1] & 0xc0) == 0x80) {
+            trunc_at--;
+            g_assert(trunc_at > 0);
+        }
 
         trunc_at--;
+        g_assert(trunc_at > 0);
+
         strcpy(p+trunc_at, "...");
         cairo_text_extents (cr, p, &extents);
     }
@@ -1267,6 +1279,7 @@ osd_render_coordinates(osm_gps_map_osd_t *osd)
 
     /* first fill with transparency */
 
+    g_assert(priv->coordinates.surface);
     cairo_t *cr = cairo_create(priv->coordinates.surface);
     cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
     //    cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 0.5);
@@ -1325,6 +1338,7 @@ osd_render_nav(osm_gps_map_osd_t *osd)
         return;
 
     /* first fill with transparency */
+    g_assert(priv->nav.surface);
     cairo_t *cr = cairo_create(priv->nav.surface);
     cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
     cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.0);
@@ -1527,6 +1541,7 @@ osd_render_crosshair(osm_gps_map_osd_t *osd)
     priv->crosshair.rendered = TRUE;
 
     /* first fill with transparency */
+    g_assert(priv->crosshair.surface);
     cairo_t *cr = cairo_create(priv->crosshair.surface);
     cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
     cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 0.0);
@@ -1583,6 +1598,7 @@ osd_render_scale(osm_gps_map_osd_t *osd)
     float m_per_pix = osm_gps_map_get_scale(OSM_GPS_MAP(osd->widget));
 
     /* first fill with transparency */
+    g_assert(priv->scale.surface);
     cairo_t *cr = cairo_create(priv->scale.surface);
     cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
     cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.0);
@@ -1725,6 +1741,7 @@ osd_render_controls(osm_gps_map_osd_t *osd)
 #endif
 
     /* first fill with transparency */
+    g_assert(priv->controls.surface);
     cairo_t *cr = cairo_create(priv->controls.surface);
     cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
     cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.0);
