@@ -1248,6 +1248,14 @@ osm_gps_map_map_redraw (OsmGpsMap *map)
 {
     OsmGpsMapPrivate *priv = map->priv;
 
+    /* on diablo the map comes up at 1x1 pixel size and */
+    /* isn't really usable. we'll just ignore this ... */
+    if((GTK_WIDGET(map)->allocation.width < 2) ||
+       (GTK_WIDGET(map)->allocation.height < 2)) {
+        printf("not a useful sized map yet ...\n");
+        return FALSE;
+    }
+
     priv->idle_map_redraw = 0;
 
     /* don't redraw the entire map while the OSD is doing */
